<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Teknisa\MAN\Utilities;
use \Estoque\Entities\Lanctoestoq;
use Zeedhi\Framework\DataSource\DataSet; 
use Zeedhi\Framework\DataSource\FilterCriteria;

class RetiradaProdutoAcabadoController extends  \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $utilService, $crudService, $retiradaService, $logTela, $irregularidadeEstoque, $entityManager, $encontrouIrreg, $exclui;
    
    protected $stCtrlMovEst, $stNrLancSaid, $validacoesEstoqueService;
    
    /*
        Será gerado um lançamento de entrada do produto acabado 
        e saída referente aos produtos pertencentes a composição.
    */
       
    const dataSourcePrincipal = "orm00003_retiradaprodutoacabado";
    const dataSourceProdutos = "orm00003_retiradaprodutoacabado_produtos";
    const dataSourceProdutosComposicao = "orm00003_retiradaprodutoacabado_produtos_com_composicao";
    const dataSourceProdComp = "orm00003_retiradaprodutoacabado_produtoscomposicao";
    const dataSourceProdEsto = "orm00003_retiradaprodutoacabado_produtoscomposicao_estoque";
    
    const msgRet = "Retirada para Produto Acabado: ";
    const semAtu = "Lanç. Produto Acabado (Sem atualização estoque): ";
    const qurExe = "Query executada: ";
    
    const SALVA_LOG = true;
    const LOG_QUERY = true;

    public function __construct ($crudService, $parameterBag, Utilities\Util $utilService, $retiradaService, $validacoesEstoqueService) {
        $this->utilService      = $utilService;
        $this->parameterBag     = $parameterBag;
        $this->crudService      = $crudService;
        $this->retiradaService  = $retiradaService;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->exclui           = false;
        $this->logTela          = array(); 
        $this->irregularidadeEstoque = array();
        $this->encontrouIrreg = false;
    }
    
    public function findProdutos (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourceProdutos;
        
        $requestParams = $request->getParameters();
        
        $container = $requestParams['origin']['containerName'];
        
        $container == 'orm00003_retiradaprodutoacabado'? $this->dataSourceName = self::dataSourceProdutosComposicao : self::dataSourceProdutos;
        
        $this->parameterBagColumns = array("CDNVPRODUTO", "CDSEGMER", "CDFILIAL");
        $conditions = $this->utilService->getParamsAsArray($request);
        
        $likeConditions = $this->getLikeConditions($request);
        array_merge($conditions, $likeConditions);
        
        //$newRequest = $this->gerarNovoRequest($conditions, $request);
        $newRequest = $request;
        
        parent::find($newRequest, $response);
    }
    
    public function getLikeConditions($request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = array();
        foreach ($conditions as $condition) {
            if($condition['operator'] === 'LIKE_ALL' ||
               $condition['operator'] === 'MAPPED_LIKE_ALL')
                array_push($result, $condition);
        }
        return $result;
    }
                  
    public function findPrincipal (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::dataSourcePrincipal;
        
        $conditions = $this->utilService->getParamsAsArray($request);
        $this->parameterBagColumns = array("CDFILIAL", "CDNVPRODUTO", "CDSEGMER", "CDARVPROD", 
    "DTINICIAL", "DTFINAL", "DATA", "ALL_FIELDS");
        
       
        if(!array_key_exists('CDARVPROD', $conditions)) { 
            $conditions['CDARVPROD'] = 'T';    
        }else{
            $conditions['CDARVPROD'] = str_replace("%", "", $conditions['CDARVPROD']);
        }        
        
        if (array_key_exists('NRCTRLMOVEST|NRLANESTENTR|NRLANESTSAID|DTMOVIESTQ|CDARVPROD|NMPRODUTO', $conditions)) {
            $conditions['ALL_FIELDS'] = $conditions['NRCTRLMOVEST|NRLANESTENTR|NRLANESTSAID|DTMOVIESTQ|CDARVPROD|NMPRODUTO'];
            unset($conditions['NRCTRLMOVEST|NRLANESTENTR|NRLANESTSAID|DTMOVIESTQ|CDARVPROD|NMPRODUTO']);
            
            $conditions['ALL_FIELDS'] = $conditions['ALL_FIELDS'][0]['_ALL'];
            $conditions['ALL_FIELDS'] = str_replace("%", "", $conditions['ALL_FIELDS']);
        } else {
            $conditions['ALL_FIELDS'] = 'T';
        }
        
        if(array_key_exists('DTRETIRADA', $conditions)){
            $conditions['DTINICIAL'] = $conditions['DTRETIRADA'][0];
            $conditions['DTFINAL'] = $conditions['DTRETIRADA'][1];
            unset($conditions['DTRETIRADA']);
            $conditions['DATA'] = null;
        }else{
            $conditions['DATA'] = 'T';
            $conditions['DTINICIAL'] = '';
            $conditions['DTFINAL'] = '';
        }

        $newRequest = $this->gerarNovoRequest($conditions, $request);
        
        parent::find($newRequest, $response); 
    }
    
    public function getParametrosFilial (DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        
        $params = array (
            ":CDFILIAL" => $conditions["CDFILIAL"]    
        );
        
        $resposta = $this->retiradaService->executarQuerySelect(self::PARAMETROS_FILIAL, $params);
        $response->addDataSet(new DataSet('parametros', array('dataset'=>$resposta)));
    }
    
    public function findProdutosCompostos (DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
            
        if (array_key_exists("NRLANCESTQ", $conditions)) {
            //tela de retirada nao passa por aqui
            $this->dataSourceName = self::dataSourceProdEsto;
            $this->parameterBagColumns = array("CDFILIAL", "NRLANCESTQ");
            parent::find($request, $response);
        } else {
            $this->dataSourceName = self::dataSourceProdComp;
            $params = $this->buildParameters($request);
            if (!empty($params[":DTMOVIESTQ"])) {
                list($dd,$mm,$yy) = explode('/',$params[":DTMOVIESTQ"]);
    			$date = sprintf('%s-%s-%s',$yy,$mm,$dd);
    			$newdate = strtotime ( '+1 day' , strtotime($date)) ;
                $params[":DTPOSICAO"] = date('d/m/Y', $newdate);
            }
            
            $produtos = $this->retiradaService->executarQuerySelect(self::PRODUTOS_COMP_L, $params); //composicao Local
            
            if(is_array($produtos) && count($produtos) == 0 ){
                $produtos = $this->retiradaService->executarQuerySelect(self::PRODUTOS_COMP, $params); //composicao Geral
            }
            if(is_array($produtos) && count($produtos) == 0 ){  
                $produtos = $this->retiradaService->executarQuerySelect(self::PRODUTOS_ESTOQ, $params); //Produto estoque
            }
            
            foreach ($produtos as &$produto) {
                
                $params[':CDPRODESTO'] = $params[':CDPRODUTO'] = $produto['CDPRODESTO'];
                $data = $this->retiradaService->executarQuerySelect(self::ADQMAXDTPOSEST, $params);
                $params[':DTPOSICAO'] = $data[0]['DTPOSIESTQ'];
                $params[':DTCUSTO'] = $params[":DTMOVIESTQ"];
                $params[':CDFILIPROC'] = $params[':CDFILIAL'];
                $params[':NRLANCESTQ'] = null;
                
                $data = $this->retiradaService->executarQuerySelect(self::ADQPOSESTLANC, $params);
   
                if (!empty($data)) {
                    $produto['VRUNILANCTO'] = round($data[0]['VRCUSTOPROD'], 2);
                } else {
                    $produto['VRUNILANCTO'] = 0;
                }
            }            
            unset($produto); 
            $response->addDataSet(new DataSet($this->dataSourceName, $produtos));
        }
    }
    
    public function buildParameters (DTO\Request\Filter $request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $params = array();
        
        foreach ($conditions as $filtro) {
            $params[':'.$filtro['columnName']] = empty($filtro['value']) ? ' ' : $filtro['value']; 
        }
        
        return $params;
    }
    
    public function getDataProcessamento (DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        $dataProcessamento = $this->getDataProc($conditions["CDFILIAL"]);
        $response->addDataset(new DataSet('DTPROC', array($dataProcessamento)));
    }
    
    public function realizarOperacaoDesativado(DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if ($conditions["__is_new"]) {
            
            $cdcontador = "LANCTOEST".$conditions["CDFILIAL"];
            $conditions["NRCTRLMOVEST"] = $this->retiradaService->getNovoCodigo($cdcontador, $conditions["NRORG"]);
            
            $cdcontador = "NRTRANSESTQ".$conditions["CDFILIAL"];
            $conditions["NRLANESTENTR"] = $this->retiradaService->getNovoCodigo($cdcontador, $conditions["NRORG"]);
            $conditions["NRLANESTSAID"] = $this->retiradaService->getNovoCodigo($cdcontador, $conditions["NRORG"]);
            
            $conditions["NRLANCESTQ"] = $conditions["NRLANESTSAID"];
            $conditions["QTLANCTOEST"] = $conditions["QTTOTLANCTO"];
            $conditions["IDEVOLESTOQ"] = $conditions["IDCNTRESTOQ"];
            $conditions["DTLANCMOVI"] = $conditions["DTMOVIESTQ"];
            $conditions["NRSEQUITEM"] = '0001';
            $conditions["IDTIPOMOVI"] = '3';
            $conditions["IDORDEMCLC"] = '9';
            $conditions["VRLANCTOBRUT"] = $conditions["VRUNILANCTO"];
            $conditions["VRLANCTOEST"] = $conditions["VRUNILANCTO"];
            
            // Seguindo a especificação
            $this->retiradaService->executarQuery(self::INSERT_LANCAMENTO_SAIDA, $this->gerarParametros('lancarEntradaSaida', $conditions));
            
            
            $this->retiradaService->executarQuery(self::INSERT_LANCAMENTO_ENTRADA, $this->gerarParametros('lancarEntradaSaida', $conditions));
            $this->retiradaService->executarQuery(self::INSERT_LANCEMENTO_ENTRADA_ITEM2, $this->gerarParametros('lancarEntradaSaidaItem2', $conditions));
            // $this->retiradaService->executarQuery(self::INSERT_LANCEMENTO_ENTRADA_ITEM, $this->gerarParametros('lancarEntradaSaidaItem', $conditions));
            // $this->retiradaService->executarQuery(self::INSERT_ITEM_ACABADO, $this->gerarParametros('inserirItemAcabado', $conditions));
            $this->retiradaService->executarQuery(self::INSERT_RELACIONAMENTO, $this->gerarParametros('atualizarControleEstoque', $conditions));
            
        
            $novoConditions = array (
                "CDFILIAL"    => $conditions["CDFILIAL"],
                "NRLANCESTQ"  => $conditions["NRLANESTSAID"]
            );
            
            $newRequest = $this->gerarNovoRequest($novoConditions, $request);
            $resposta   = $this->findProdutosCompostos($newRequest, $response);
            $reposta    = $resposta->getDataSet()->getRows();
            
            foreach ($resposta as $produto) {
                $conditions["CDPRODUTO"] = $produto["CDPRODUTO"];
                $conditions["NRSEQUITEM"] = $produto["NRSEQUITEM"];
                $conditions["VRLANCTOBRUT"] = $produto["VRLANCTOBRUT"];
                $conditions["VRLANCTOEST"] = $produto["VRLANCTOEST"];
                $conditions["VRFATOCONV"] = $produto["VRFATOCONV"];
                $conditions["IDTIPOMOVI"] = '2';
                $conditions["IDORDEMCLC"] = '2';
                
                $this->retiradaService->executarQuery(self::INSERT_LANCAMENTO_SAIDA, $this->gerarParametros('lancarEntradaSaida', $conditions));
                $this->retiradaService->executarQuery(self::INSERT_LANCAMENTO_SAIDA_ITEM, $this->gerarParametros('lancarEntradaSaidaItem', $conditions));
                $this->retiradaService->executarQuery(self::INSERT_LANCAMENTO_SAIDA_ITEM2, $this->gerarParametros('lancarEntradaSaidaItem2', $conditions));
        
            }
            
            
        } else {
            // Atualiza Item Acabado
            $this->retiradaService->executarQuery(self::UPDATE_ITEM_ACABADO, $this->gerarParametros('atualizarItemAcabado', $conditions));
            // Atualiza Controle Estoque
            $this->retiradaService->executarQuery(self::UPDATE_CTRL_ESTOQUE, $this->gerarParametros('atualizarControleEstoque', $conditions));
            
        }
        
        $this->retiradaService->atualizarEstoque($this->gerarParametros('atualizarEstoque', $conditions));
        
    }
    
    public function getDataProc($cdfilial) {
        $params[":CDFILIAL"] = $cdfilial;
        $dtproc = $this->retiradaService->executarQuerySelect(self::DATAPROC, $params);
        
        if(empty($dtproc)) {
            $dtproc = date('d/m/y');
        } else {
            $dtproc = $dtproc[0]['DTPROCESSA'];
        }
        
        return $dtproc;
    }

    public function realizarOperacaoDes2 (DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);

        $produtos = $conditions["PRODUTOSACAB"];
        $result = array();
        $adqParamFilial = $this->retiradaService->executarQuerySelect(self::ADQPARAMFILIAL, $this->gerarParametros('adqParamFilial', $conditions));
        
        foreach ($produtos as $produto) {
            $result[] = $this->retiradaService->consultaCustoQuantidadeProduto($produto["CDFILIAL"], 
                                    $produto["CDPRODESTO"], $conditions["DTMOVIESTQ"], $adqParamFilial[0]["CDTIPCUSTEST"], $produto["CDALMOXARIFE"]);
        }
        
    }
    
    public function realizarOperacao(DTO\Request\Row $request, DTO\Response $response) {
        
        $conditions = $this->utilService->getParamsAsArray($request);
        
        $this->entityManager = $this->retiradaService->entityManager;
        $this->entityManager->getConnection()->beginTransaction();
        $this->validacoesEstoqueService->entityManager = $this->entityManager;
        
        $stTipoReti = $conditions["CDTIPORETI"];
        $cdfilial = $conditions["CDFILIAL"];
        $conditions["DATAPROC"] = $this->getDataProc($cdfilial);
        $update = false;
        
        $this->stNrLancSaid = "";
        $stNrLancEntr = "";
        
        $tolerancia = false;
        $efetuarRollback = true;
        
        if (!empty($conditions["ROLLBACK"]) && $conditions["ROLLBACK"] == 'N') {
            $efetuarRollback = false;    
        }
        
        //$tolerancia = $conditions["TOLERANCIA"];
        
        if (!empty($conditions["NRLANESTSAID"])) $this->stNrLancSaid = $conditions["NRLANESTSAID"];
        
        if (!empty($conditions["NRLANESTENTR"])) $stNrLancEntr = $conditions["NRLANESTENTR"];
        
        $params[":CDFILIAL"] = $conditions["CDFILIAL"];
        $parametrosUnidade = $this->retiradaService->executarQuerySelect(self::PARAM_FILIAL, $params);
        
        if (!empty($conditions["EXCLUI"]) && $conditions["EXCLUI"] == 'S') $this->exclui = true;
        
        if (!$this->exclui) {
            
            $adqExisteCtrl = $this->retiradaService->executarQuerySelect(self::ADQEXISTECTRL, $this->gerarParametros('adqExisteCtrl', $conditions));
            
            //if adqExisteCtrl.EOF then begin - Caso nao tenha nenhum resultadoo
            if (empty($adqExisteCtrl)) {
                $cdcontador = "LANCTOEST".$conditions["CDFILIAL"];
                $this->stNrLancSaid  = $this->retiradaService->getNovoCodigo($cdcontador, $conditions["NRORG"]);
                $stLancEstSaid = $this->stNrLancSaid;
                $conditions["NRLANESTSAID"] = $this->stNrLancSaid;

                $cdcontador = "NRTRANSESTQ".$conditions["CDFILIAL"];
                $this->stCtrlMovEst = $this->retiradaService->getNovoCodigo($cdcontador, $conditions["NRORG"]);

                $cdcontador = "LANCTOEST".$conditions["CDFILIAL"];
                $stNrLancEntr = $this->retiradaService->getNovoCodigo($cdcontador, $conditions["NRORG"]);
                $conditions["NRLANESTENTR"] = $stLancEstEntr = $stNrLancEntr;
                
                $parametros = $this->gerarParametros('adqInsertLancto', $conditions);
                $parametros[":NRLANCESTQ"] = $this->stNrLancSaid; 
                $parametros[":IDTPLANCTO"] = '4';
        
                $this->retiradaService->executarQuery(self::ADQINSERTLANCTO, $parametros);
                
                $this->addQueryNoLog('adqInsertLancto - Parâmetros: CDFILIAL = '.$conditions['CDFILIAL'].' NRLANCESTQ = '.$parametros[":NRLANCESTQ"]);
                
                $conditions["NRLANESTENTR"] = $stNrLancEntr;
                $conditions["NRCTRLMOVEST"] = $this->stCtrlMovEst;
                
                $conditions["NRLANCESTQ"] = $conditions["NRLANESTSAID"];
                $conditions["QTLANCTOEST"] = $conditions["QTTOTLANCTO"];
                $conditions["IDEVOLESTOQ"] = $conditions["IDCNTRESTOQ"];
                $conditions["DTLANCMOVI"] = $conditions["DTMOVIESTQ"];
                
                $conditions["NRSEQUITEM"] = '0001';
                $conditions["IDTIPOMOVI"] = '3';
                $conditions["IDORDEMCLC"] = '9';

                if(!isset($conditions["VRUNILANCTO"])) {
                    $conditions["VRUNILANCTO"] = null;
                }
                
                $conditions["VRLANCTOBRUT"] = $conditions["VRUNILANCTO"];
                $conditions["VRLANCTOEST"] = $conditions["VRUNILANCTO"];
                
                //dtmRotiEstoque.AtualizaProdLocal(oleItemRetAcab, 'S');
                $adqProdFili = $this->retiradaService->executarQuerySelect(self::ADQPRODFILI, $this->gerarParametros('adqProdFili', $conditions));

                // dtmRotiEstoque.AtualizaProdLocal(oleItemAcabado, 'E');
    
                
                $parametros = $this->gerarParametros('adqInsertLancto', $conditions);
                if ((count($adqProdFili) > 0) && $adqProdFili[0]["IDCNTRESTOQ"] != 'S' ) {
                    $parametros[':DSLANCESTQ'] = self::semAtu.$this->stCtrlMovEst;
                }
                $parametros[":NRLANCESTQ"] = $stLancEstEntr;
                $parametros[":IDTPLANCTO"] = '2';
                $this->retiradaService->executarQuery(self::ADQINSERTLANCTO, $parametros);
                $this->addQueryNoLog('adqInsertLancto - Parâmetros: CDFILIAL = '.$parametros[':CDFILIAL'].' NRLANCESTQ = '.$parametros[":NRLANCESTQ"]);
                    
                $this->addLog(" ");
                $this->atualizaProdLocal($conditions, 'S');
                $this->atualizaProdLocal($conditions, 'E');
                // $this->retiradaService->executarQuery(self::INSERT_LANCAMENTO_SAIDA, $this->gerarParametros('lancarEntradaSaida', $conditions));
                // $this->retiradaService->executarQuery(self::INSERT_LANCEMENTO_ENTRADA_ITEM2, $this->gerarParametros('lancarEntradaSaidaItem2', $conditions));

                $this->retiradaService->executarQuery(self::INSERT_RELACIONAMENTO, $this->gerarParametros('atualizarControleEstoque', $conditions));
                $this->addQueryNoLog('atualizarControleEstoque');
                
            } else {
                $update = true;
                $stNrLancEntr = $conditions["NRLANESTENTR"];
                $this->stCtrlMovEst = $conditions["NRCTRLMOVEST"];
                
                $parametros[":CDFILIAL"] = $conditions["CDFILIAL"];
                $parametros[":NRLANCESTQ"] = $conditions["NRLANESTENTR"];
        
                $this->retiradaService->executarQuery(self::ACMDELITLANCTOEST, $parametros);
                $this->addLog("acmDelLanctoEstq -> ".$parametros[":NRLANCESTQ"]);
                
                $parametros[":NRLANCESTQ"] = $conditions["NRLANESTSAID"];            
                $this->retiradaService->executarQuery(self::ACMDELITLANCTOEST, $parametros);
        
                $this->addLog("acmDelItLanctoEst -> ".$parametros[":NRLANCESTQ"]);
                
                $this->addLog(" ");
                $this->atualizaProdLocal($conditions, 'S');
                $this->atualizaProdLocal($conditions, 'E');
    
                $this->retiradaService->executarQuery(self::ADQUPDATELANCTOESTOQ, $this->gerarParametros('adqUpdateLanctoEstoq', $conditions));
    
                $this->retiradaService->executarQuery(self::ADQUPDATEDTLANCMOVI, $this->gerarParametros('adqUpdateDtLancMovi', $conditions));
                $parametros = $this->gerarParametros('adqUpdateLanctoEstoq', $conditions);
                $parametros["NRLANCESTQ"] = $conditions["NRLANESTENTR"];
                $this->retiradaService->executarQuery(self::ADQUPDATELANCTOESTOQ, $parametros);
                
                $parametros = $this->gerarParametros('adqUpdateDtLancMovi', $conditions);
                $parametros["NRLANCESTQ"] = $conditions["NRLANESTENTR"];
                $parametros["DTLANCESTQ"] = $conditions["DTMOVIESTQ"];
                $this->retiradaService->executarQuery(self::ADQUPDATELANCTOESTOQ, $parametros);
                
                // Atualiza Item Acabado
                $this->retiradaService->executarQuery(self::UPDATE_ITEM_ACABADO, $this->gerarParametros('atualizarItemAcabado', $conditions));
                // Atualiza Controle Estoque
                $this->retiradaService->executarQuery(self::UPDATE_CTRL_ESTOQUE, $this->gerarParametros('atualizarControleEstoque', $conditions));
                
                $this->retiradaService->executarQuery(self::UPDATE_CTRL_ESTOQUE, $this->gerarParametros('atualizarControleEstoque', $conditions));

                
            }
            $CDFILIAL = $conditions["CDFILIAL"];
            $lanctoestoqSaid = Lanctoestoq::findByNrlancestqAndCdfilial($this->stNrLancSaid, $CDFILIAL)[0];
            $lanctoestoqEntr = Lanctoestoq::findByNrlancestqAndCdfilial($stNrLancEntr, $CDFILIAL)[0]; 
        } else {
            $this->stCtrlMovEst = "";
            $CDFILIAL = $conditions["CDFILIAL"];
            $lanctoestoqSaid = Lanctoestoq::findByNrlancestqAndCdfilial($this->stNrLancSaid, $CDFILIAL)[0];
            $lanctoestoqEntr = Lanctoestoq::findByNrlancestqAndCdfilial($stNrLancEntr, $CDFILIAL)[0]; 
            
            $parametros = array(
                ":CDFILIAL"    => $conditions["CDFILIAL"]
            );
            
            $this->addLog("Excluindo produto acabado.");
            
            $parametros[":NRCTRLMOVEST"] = $conditions["NRCTRLMOVEST"];
            $this->retiradaService->executarQuery(self::ACMDELCTRLMOVIESTQ, $parametros);
            $this->addLog("acmDelCtrlMoviEstq -> ".$parametros[":NRCTRLMOVEST"]);
            
            $parametros[":NRLANCESTQ"] = $conditions["NRLANESTENTR"];
            $this->retiradaService->executarQuery(self::ACMDELLANCTOESTQ, $parametros);
            $this->addLog("acmDelLanctoEstq -> ".$parametros[":NRLANCESTQ"]);
            
            $parametros[":NRLANCESTQ"] = $conditions["NRLANESTSAID"];            
            $this->retiradaService->executarQuery(self::ACMDELITLANCTOEST, $parametros);
            $this->addLog("acmDelItLanctoEst -> ".$parametros[":NRLANCESTQ"]);
            
         /*   $this->retiradaService->executarQuery(self::ACMDELITLANCTOEST, $parametros);
            $this->addLog("acmDelItLanctoEst -> ".$parametros[":NRLANCESTQ"]);
            $this->addLog("acmDelLanctoEstq -> ".$parametros[":NRLANCESTQ"]);
            
            $this->retiradaService->executarQuery(self::ACMDELITLANCTOEST, $parametros);
            $this->addLog("acmDelItLanctoEst -> ".$parametros[":NRLANCESTQ"]);
            $this->addLog("acmDelLanctoEstq -> ".$parametros[":NRLANCESTQ"]);
            
            $parametros[":NRLANCESTQ"] = $conditions["NRCTRLMOVEST"];
            $this->retiradaService->executarQuery(self::ACMDELLANCTOESTQ, $parametros);
            $this->addLog("acmDelLanctoEstq -> ".$parametros[":NRLANCESTQ"]);
            
            $this->addQueryNoLog('acmDelItLanctoEst');
            $this->addQueryNoLog('acmDelLanctoEst');*/
        }
        
        $return = array();
        // INICIO MÉTODO ESTOQUE
        
        $this->verifyEstoque($lanctoestoqSaid, $CDFILIAL);
        $this->verifyEstoque($lanctoestoqEntr, $CDFILIAL);
            
        if ($this->encontrouIrreg) {
            $this->entityManager->getConnection()->rollBack();
            $response->addDataset(new DataSet('irregularidades', [true]));
            // $response->addDataset(new DataSet('produtos_irregulares', $this->irregularidadeEstoque));
        } else {
            $this->entityManager->getConnection()->commit();
        }
        
        $response->addDataset(new DataSet('log', $this->logTela));
        
    }
    
    public function verifyEstoque($lanctoestoq, $CDFILIAL) {
        
        $lancamentoEstoque = $lanctoestoq->getNrlancestq();
        $this->validacoesEstoqueService->setLanctoestoq($lanctoestoq);
        
        try {
            $this->validacoesEstoqueService->atualizaEstoque($CDFILIAL, $lancamentoEstoque);
        } catch (\Exception $e) {
            $this->encontrouIrreg = true;
        }
        
    }
    
    public function setLocaisDefault(&$conditions){
        $arrLocais = array('CDLOCALESTOQ', 'CDALMOXARIFE', 'NRSUBLOTE', 'NRLOTEESTQ');
        foreach($arrLocais as $local){
            if(!isset($conditions[$local]))
                $conditions[$local] = ' ';
            elseif(empty($conditions[$local]))
                $conditions[$local] = ' ';
        }
    }
    
    public function atualizaProdLocal($conditions, $stEntrSaid) {
        
        // function ResultLote(stIdEntrSaid: String): String;
        $stLote = '';
        
        //cria e seta valores default para locais de estoque caso nao existam
        $this->setLocaisDefault($conditions);
        
        if ($stEntrSaid == 'S') {
            $produtos = $conditions["PRODUTOSACAB"]; 
            $queryInsert = self::INSERT_LANCAMENTO_SAIDA_ITEM2;
            $NRLANCESTQ = $this->stNrLancSaid;
        } else {
            if(!isset($conditions["VRTOTLANCTO"]) || !isset($conditions["VRUNILANCTO"])) {
                $conditions["VRTOTLANCTO"] = null;
                $conditions["VRUNILANCTO"] = null;
            }
            //preenche valor total conforme produtos acabados caso nao informado
            if(is_null($conditions["VRTOTLANCTO"]) || is_null($conditions["VRUNILANCTO"])){
                $conditions["VRUNILANCTO"] = 0;
                foreach($conditions["PRODUTOSACAB"] as $prodacab){
                    $conditions["VRUNILANCTO"] += $prodacab["VRUNILANCTO"];
                }
                $conditions["VRTOTLANCTO"] = $conditions["VRUNILANCTO"] * $conditions["QTLANCTOEST"];
            }
            
            $produto = array (
                "CDFILIAL"      => $conditions["CDFILIAL"],
                "CDPRODUTO"     => $conditions["CDPRODUTO"],
                "NMPRODUTO"     => $conditions["NMPRODUTO"],
                "CDPRODESTO"    => $conditions["CDPRODESTO"],
                "QTLANCTOEST"   => $conditions["QTLANCTOEST"],
                "VRTOTLANCTO"   => $conditions["VRTOTLANCTO"],
                "QTTOTLANCTO"   => $conditions["QTTOTLANCTO"],
                "VRUNILANCTO"   => $conditions["VRUNILANCTO"],
                "CDALMOXARIFE"  => empty($conditions["CDALMOXENTR"]) ? " " : $conditions["CDALMOXENTR"],
                "VRFATOCONV"    => $conditions["VRFATOCONV"],
                "CDLOCALESTOQ"  => empty($conditions['CDLOCALESTOQ']) ? null : $conditions["CDLOCALESTOQ"],
                "NRLOTEESTQ"    => empty($conditions["NRLOTEESTQ"]) ? null : $conditions["NRLOTEESTQ"],
                "NRSUBLOTE"     => empty($conditions["NRSUBLOTE"]) ? null : $conditions["NRSUBLOTE"]
            );
            $produtos = array();
            $produtos[] = $produto;
            $queryInsert = self::INSERT_LANCEMENTO_ENTRADA_ITEM2;
            $NRLANCESTQ = $conditions["NRLANESTENTR"];
        }
        
        $i = 1;
        $this->addLog("atualizaProdLocal -> ".$stEntrSaid);
        try { 
            // with cdsItemLanc do begin
            
            $stCampoProduto = "CDPRODESTO";
            // $stCampoProduto = "CDPRODMOVI";
                
            /*try
                stCampoProduto := FieldByName('CDPRODMOVI').FieldName;
            except
                stCampoProduto := 'CDPRODESTO';
            end;*/

            $adqParamFilial = $this->retiradaService->executarQuerySelect(self::ADQPARAMFILIAL, $this->gerarParametros('adqParamFilial', $conditions));
            $this->addLog("IDCTRLESTQ: ".$adqParamFilial[0]['IDCTRLESTQ']);
            $this->addLog("CDFILIAL: ".$conditions['CDFILIAL']);
            
            if (($adqParamFilial[0]['IDCTRLESTQ'] != 'N') and ($conditions['CDFILIAL'] != null)) {
                try {
                    foreach ($produtos as &$item) {
                        $string = "Produto: ".$item[$stCampoProduto].". ";
                        // removido do if:  and $this->produtoControlaEstoque($conditions["CDFILIAL"], $item[$stCampoProduto])
                        if ($item[$stCampoProduto] != null and $item["CDFILIAL"] != null) {

                            $qtlantoest = $item['QTLANCTOEST'];
                            $parametros = array (
                                ":CDFILIAL"       => $item["CDFILIAL"],
                                ":CDALMOXARIFE"   => empty($item["CDALMOXARIFE"]) ? " " : $item["CDALMOXARIFE"],
                                ":CDLOCALESTOQ"   => empty($item['CDLOCALESTOQ']) ? " " : $item["CDLOCALESTOQ"],
                                ":CDPRODUTO"      => $item["CDPRODUTO"]
                            );
                            $this->retiradaService->executarQuery(self::ACMINSERTPRODLCESTOQUE, $parametros);
                            $string .= "acmInsertProdLcEstoque ";
        
                            $stLote = $this->resultLote($stEntrSaid, $conditions);

                            $parametros = array (
                                ":CDFILIAL"    => $conditions["CDFILIAL"],
                                ":NRLOTEESTQ"  => $this->resultValue($stLote),
                                ":NRSUBLOTE"   => $this->resultValue(empty($item["NRSUBLOTE"]) ? null : $item["NRSUBLOTE"]),
                                ":CDPRODUTO"   => $item["CDPRODUTO"]
                            );
        
                            $adqVerifLote = $this->retiradaService->executarQuerySelect(self::ADQVERIFLOTE, $parametros);
                            
                            if (sizeof($adqVerifLote) == 0 && $stEntrSaid == 'E') {
                                $parametros = array (
                                    ":CDFILIAL"    => $conditions["CDFILIAL"],
                                    ":NRLOTEESTQ"  => $this->resultValue($stLote),
                                    ":NRSUBLOTE"   => $this->resultValue(empty($item["NRSUBLOTE"]) ? " " : $item["NRSUBLOTE"]),
                                    ":CDPRODUTO"   => $item["CDPRODUTO"]
                                );
                                $this->retiradaService->executarQuery(self::ACMINSERTLOTE, $parametros);
                                $string .= "acmInsertLote";
                            }
                            
                            
                            if(!isset($item['REPLICADO'])){
                                $qtde = $item['QTLANCTOEST'];
                                $item['VRFATOCONV'] = $item['VRFATOCONV']?? 0;
                                $fatoConv = $item['VRFATOCONV'];
                                $qtdeLancTotal = $qtde * $fatoConv;
                                $result = $this->retiradaService->consultaCustoQuantidadeProdutoES($conditions, $item, $adqParamFilial[0]["CDTIPCUSTEST"], $stEntrSaid);
                                
                                foreach($result as $rowCusto){
                                    if (empty($rowCusto["VRCUSTOPROD"])) {
                                        if (empty($rowCusto[0]["VRCUSTOPROD"]))
                                            $rowCusto["VRCUSTOPROD"] = 0;
                                        else
                                            $rowCusto["VRCUSTOPROD"] = $rowCusto[0]["VRCUSTOPROD"];
                                    }
                                    
                                    $result = $rowCusto;
                                    $item['NRLOTEESTQ'] = $rowCusto['NRLOTEESTQ'] ?? ' ';
                                    $item['NRSUBLOTE']  = $rowCusto['NRSUBLOTE'] ?? ' ';
                                    if(!isset($item['VRTOTLANCTO']) || is_null($item['VRTOTLANCTO']) && $stEntrSaid == 'S')
                                        $item['VRTOTLANCTO']  = $rowCusto['VRCUSTOPROD'] * $rowCusto['QTESTOQDIA'];
                                    $item['VRLANCTOEST']  = $item['VRTOTLANCTO'];
                                    $item['VRLANCTOBRUT'] = $rowCusto['VRCUSTOPROD'];
                                    $item['QTLANCTOEST']  = $item['VRFATOCONV'] == 0? 0 : $qtdeLancTotal / $item['VRFATOCONV'];
                                    if(!isset($item['VRUNILANCTO']) || is_null($item['VRUNILANCTO']) && $stEntrSaid == 'S')
                                        $item['VRUNILANCTO']  = $rowCusto['VRCUSTOPROD'];
                                    break 1;//sai do loop de custos
                                    
                                }
                            }//Fim dos custos e replicação dos produtos
                            
                            if (gettype($item["VRUNILANCTO"]) == 'string') {
                                $item["VRUNILANCTO"] = $item["VRUNILANCTO"];
                            }
                            if (!empty($item["QTTOTLANCTO"]) && gettype($item["QTTOTLANCTO"]) == 'string') {
                                $item["QTTOTLANCTO"] = $item["QTTOTLANCTO"];
                            }
                            if (gettype($item["QTLANCTOEST"]) == 'string') {
                                $item["QTLANCTOEST"] = $item["QTLANCTOEST"];
                            }
                            
                            $controla = $this->produtoControlaEstoque($parametros[":CDFILIAL"], $parametros[':CDPRODUTO']);
                            if ($controla) {
                                $IDEVOLESTOQ = 'S';
                            } else {
                                $IDEVOLESTOQ = 'N';
                            }
                            
                            $parametros = array (
                               ":CDFILIAL"      => $conditions["CDFILIAL"],
                               ":NRSEQUITEM"    => str_pad($i, 4, "0", STR_PAD_LEFT),
                               ":NRLANCESTQ"    => $NRLANCESTQ,
                               ":CDPRODUTO"     => $parametros[":CDPRODUTO"],
                               ":NMPRODUTO"     => $item["NMPRODUTO"],
                               ":CDPRODMOVI"    => $item["CDPRODESTO"],
                               ":VRUNILANCTO"   => $item["VRUNILANCTO"],
                               ":VRTOTLANCTO"   => $item["VRTOTLANCTO"],
                               ":QTTOTLANCTO"   => $item["QTLANCTOEST"],
                               ":QTLANCTOEST"   => floatval($item["QTLANCTOEST"]) * floatval($item['VRFATOCONV']),
                               ":DTLANCMOVI"    => $conditions["DTLANCMOVI"],
                               ":VRLANCTOBRUT"  => $stEntrSaid == 'E'? $conditions["VRLANCTOBRUT"] : $item["VRLANCTOBRUT"],
                               ":VRLANCTOEST"   => $stEntrSaid == 'E'? $conditions["VRLANCTOEST"]  : $item["VRLANCTOEST"],
                               ":CDFILIMOVI"    => $conditions["CDFILIAL"],
                               ":CDALMOXARIFE"  => $stEntrSaid == 'E'? $conditions['CDALMOXARIFE'] : (empty($item["CDALMOXARIFE"]) ? " " : $item["CDALMOXARIFE"]),
                               ":CDLOCALESTOQ"  => $stEntrSaid == 'E'? $conditions['CDLOCALESTOQ'] : (empty($item['CDLOCALESTOQ']) ? " " : $item["CDLOCALESTOQ"]),
                               ":NRLOTEESTQ"    => $stEntrSaid == 'E'? $this->resultValue($stLote) : $item['NRLOTEESTQ'],
                               ":NRSUBLOTE"     => $stEntrSaid == 'E'? $conditions['NRSUBLOTE'] : ($this->resultValue(empty($item["NRSUBLOTE"]) ? " " : $item["NRSUBLOTE"])),
                               ":IDEVOLESTOQ"   => $IDEVOLESTOQ,
                               ":CDTIPORETI"    => $conditions["CDTIPORETI"] 
                            );
                            if ($stEntrSaid == 'E') {
                                $parametros[":VRUNILANCTO"] = $conditions["VRUNILANCTO"];
                                $parametros[":VRTOTLANCTO"] = $item["VRTOTLANCTO"];
                            }
                            $this->retiradaService->executarQuery($queryInsert, $parametros);

                            $this->addLog($string);
                            $i++;
                        } else {
                            $this->addLog("Produto não inserido corretamente.");
                        }
                                           }
                } catch (\Exception $e) {
                    throw $e;
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
        
        $this->addLog(" ");
    }
    
    private function recEstoque($arsParametros, $dtInicial, $dtFinal, $conditions) {
          try {
                $this->addLog("recEstoque Parâmetros: 0: ".$arsParametros[0]." 1: ".$arsParametros[1]." 2: ".$arsParametros[2]);
                $this->addLog("recEstoque Datas-> Inicial: ".$dtInicial." Final: ".$dtFinal);
                $parametros = $this->gerarParametros('atualizarEstoque', $conditions);
              
                $params = array(
                    "P_FILIMOVI"    => $arsParametros[0],
                    "P_FILILANCTO"  => $arsParametros[1],
                    "P_DTINICIAL"   => $dtInicial,
                    "P_DTFINAL"     => $dtFinal
                );

        
                if (sizeof($arsParametros) > 1) {
                    if ($arsParametros[2] = '')
                       $params["P_NRLANCTO"] = null;
                    else 
                       $params["P_NRLANCTO"] = $arsParametros[2];
                } else 
                    $params["P_NRLANCTO"] = null;
        
                 $this->retiradaService->atualizarEstoque($params);
                 $this->addQueryNoLog('ATUALIZA_ESTOQUE');
            } catch (\Exception $e) {
                throw $e;
            }
    }
    
    private function resultValue ($stFieldValue) {
        $result = $stFieldValue;
        if (($stFieldValue = '') or ($stFieldValue = null)) $result = ' ';
        return $result;
    }
    
    private function resultLote($stIdEntrSaid, $conditions) {
        $result = " ";
        $stCampoProduto = $conditions["CDPRODUTO"];
        if (empty($conditions["NRLOTEESTQ"])) {
             if ($this->utilizaControleLote($conditions["CDFILIAL"])) {
                if ($stIdEntrSaid = 'E') {
                   try {
                    //   $result = dtmProxCodigo.PrxLote(cdsItemLanc.FieldByName('CDFILIMOVI').AsString, $stCampoProduto);
                   } catch (\Exception $e) {
                      throw $e; 
                   }
                } else {
                    $adqLote = $this->retiradaService->executarQuerySelect(self::ADQLOTE, $this->gerarParametros('adqLote', $conditions));
                    $result  = $adqLote[0]['NRLOTEESTQ'];
                }
            } 
        } else {
            $result = $conditions['NRLOTEESTQ'];
        }
        
        return $result;
    }
    
    private function produtoControlaEstoque($stFilial, $stProduto) {
        $resultado = false;
        $params = array(
            ":CDFILIAL"      => $stFilial,
            ":CDPRODUTO"     => $stProduto
        );
        
        $adqProdFili = $this->retiradaService->executarQuerySelect(self::ADQPRODFILIEST, $params);
        
        if ($adqProdFili[0]["IDCNTRESTOQ"] != 'N') {
            $resultado = true;
        }
        
        return $resultado;
    }
    
    private function utilizaControleLote($cdfilial) {
        $params[":CDFILIAL"] = $cdfilial;
        $adqParamFilial = $this->retiradaService->executarQuerySelect(self::ADQPARAMFILIAL, $params);
      
        return ($adqParamFilial[0]['IDUTILLOTE'] == 'S');
    }
    
    // Carece de atenção
    private function rollback($boRollbackIrreqEst, $arsParametros) {
        $this->encontrouIrreg = true;
        if ($boRollbackIrreqEst && !$this->exclui) {
            $this->entityManager->getConnection()->rollBack();
            /*$params = array(
                    ":CDFILIAL"   => substr($arsParametros[0], 0, 4),
                    ":NRLANCESTQ" => substr($arsParametros[0], 0, 10)
            );
            
            $adqLanctoEstoq = $this->retiradaService->executarQuerySelect(self::ADQLANCTOESTOQ, $params);
            
            
            $params["CDFILIAL"] = $adqLanctoEstoq[0]["CDFILIAL"];
            $adqParamFilial = $this->retiradaService->executarQuerySelect(self::ADQPARAMFILIAL, $params);
            */
            
            /*if ($adqParamFilial["IDPERMESTNEG"] != 'S') {
                if (TXConnection(adqParamFilial.Connection).InTransaction)
                    TXConnection(adqParamFilial.Connection).RollBack;
            }*/
            
            /*if adqParamFilialIDPERMESTNEG.Value <> 'S' then begin
                if adqParamFilial.Connection <> nil then begin
                   if TXConnection(adqParamFilial.Connection).InTransaction then begin
                      TXConnection(adqParamFilial.Connection).RollBack;
                   end;
                end;
             end;*/
            
       }
    }
    
    private function irregEstoque ($arsParametros, $boReplica, $boRollbackIrreqEst = true) {
        $stEstoqOnLine = 'T';
            
        $this->addLog("Verificando irregularidades do estoque.");
        $this->addLog("Parâmetros: ".implode(", ", $arsParametros));
        $params[":NRLANCTOORIG"] = substr($arsParametros[0], 0, 4).$arsParametros[1];
        $adqParamFili = $this->retiradaService->executarQuerySelect(self::ADQPARAMFILI, $params);
        $this->addQueryNoLog('adqParamFili');    
            
        if ($boReplica) {
            $stEstoqOnLine = 'T';
            
            if (!empty($adqParamFili)) 
                $stEstoqOnLine = 'F';

            $adqMoviLoteOrig = $this->retiradaService->executarQuerySelect(self::ADQMOVILOTEORIG, $params);
            $this->addQueryNoLog('adqMoviLoteOrig. Tamanho: '.sizeof($adqMoviLoteOrig));
            if (!empty($adqMoviLoteOrig)) {
                /*while not Eof {
                   CDSErro.Adiciona(IrreEsto, FieldByName('DSFILIAL').AsString, FieldByName('DSPRODUTO').AsString, stEstoqOnLine, '', 0);
                   Next;
                }*/
                
                foreach ($adqMoviLoteOrig as $item) {
                    $this->addLog("PRODUTO IRREGULAR! ".$item["DSPRODUTO"]);
                    $this->irregularidadeEstoque[] = "Unidade: ".$item["DSFILIAL"]." - Produto: ".$item["DSPRODUTO"];
                }
                
                $this->limpaIrreg('R');
                $this->rollback($boRollbackIrreqEst, $arsParametros);
            }
            
            $params[":NRMOVILOTE"] = substr($arsParametros[0],0,4).$arsParametros[1];
            $adqMoviLote = $this->retiradaService->executarQuerySelect(self::ADQMOVILOTE, $params);
            $this->addQueryNoLog('adqMoviLote. Tamanho: '.sizeof($adqMoviLote));
            if (empty($adqMoviLote)) {
                /*while not Eof {
                   CDSErro.Adiciona(IrreEsto, FieldByName('DSFILIAL').AsString, FieldByName('DSPRODUTO').AsString, stEstoqOnLine, '', 0);
                   Next;
                }*/
                
                foreach ($adqMoviLote as $item) {
                    $this->irregularidadeEstoque[] = "Unidade: ".$item["DSFILIAL"]." - Produto: ".$item["DSPRODUTO"];
                }
                $this->rollback($boRollbackIrreqEst, $arsParametros);
             }
             $this->limpaIrreg('L', $arsParametros);
         
        } else if (sizeof($arsParametros) == 2) {
            
            if (!empty($adqParamFili))
                $stEstoqOnLine = 'F';
    
            $params[":NRMOVILOTE"] = substr($arsParametros[0],0,4).$arsParametros[1];
            $adqMoviLote = $this->retiradaService->executarQuerySelect(self::ADQMOVILOTE, $params);
            $this->addQueryNoLog('adqMoviLote. Tamanho: '.sizeof($adqMoviLote));
            
            if (!empty($adqMoviLote)) {
                /*while not Eof {
                   CDSErro.Adiciona(IrreEsto, FieldByName('DSFILIAL').AsString, FieldByName('DSPRODUTO').AsString, stEstoqOnLine, '', 0);
                   //CDSErro.Adiciona(IrreEsto, FieldByName('CDPRODUTO').AsString);
                   Next;
                }*/
                foreach ($adqMoviLote as $item) {
                    $this->addLog("PRODUTO IRREGULAR! ".$item["DSPRODUTO"]);
                    $this->irregularidadeEstoque[] = "Unidade: ".$item["DSFILIAL"]." - Produto: ".$item["DSPRODUTO"];
                }
                $this->rollback($boRollbackIrreqEst, $arsParametros);
                $this->limpaIrreg('L', $arsParametros);
            }
        } else if (sizeof($arsParametros) == 3) {

            $params["NRMOVILOTE"] = substr($arsParametros[0],0,4).$arsParametros[1];
            $params["CDPRODUTO"] = $arsParametros[1];
 
            $adqMoviLoteProd = $this->retiradaService->executarQuerySelect(self::ADQMOVILOTEPROD, $params);
            $this->addQueryNoLog('adqMoviLoteProd. Tamanho: '.sizeof($adqMoviLoteProd));
            
            if (!empty($adqMoviLoteProd)) {
                /*while not Eof do begin
                   CDSErro.Adiciona(IrreEsto, FieldByName('CDPRODUTO').AsString);
                   Next;
                end;*/
                foreach ($adqMoviLoteProd as $item) {
                    $this->addLog("PRODUTO IRREGULAR! ".$item["DSPRODUTO"]);
                    $this->irregularidadeEstoque[] = "CDPRODUTO: ".$item["CDPRODUTO"];
                }
                $this->rollback($boRollbackIrreqEst, $arsParametros);
                $this->limpaIrreg('P', $arsParametros);
             };
          }
    }
    
    private function limpaIrreg ($char, $arsParametros) {
        switch ($char) {
            case 'R': 
                $params = array(
                    ":CDFILIALENTR" => substr($arsParametros[0], 0, 4) + $arsParametros[1]
                );
                $query = self::ACMDELETEMOVILOTEORIG;
            break;
            case 'L':
                $params = array(
                    ":NRMOVILOTE" => substr($arsParametros[0], 0, 4) + $arsParametros[1]
                );
                $query = self::ACMDELETEMOVILOTE;
            break;
            case 'P':
                $params = array(
                    ":NRMOVILOTE" => substr($arsParametros[0], 0, 4) + $arsParametros[1],
                    ":CDPRODUTO" => $arsParametros[2]
                );
                $query = self::ACMDELETEMOVILOTEPROD;
            break;
        }
        
        if (isset($query) && isset($parametros)) {
            $this->retiradaService->executarQuery($query, $parametros);
            $this->addQueryNoLog('limpaIrregEstoque');
        }
      
    }
    
    public function atualizarProduto (DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        
        $params = array(
            ":CDALMOXARIFE" => $conditions["CDALMOXENTR"],
            ":CDFILIAL"     => $conditions["CDFILIAL"], 
            ":NRLANCESTQ"   => $conditions["NRLANCESTQ"], 
            ":NRSEQUITEM"   => $conditions["NRSEQUITEM"]
        );
        
        $this->retiradaService->atualizarProduto($params);
    }
    
    public function gerarNovoRequest ($conditions, $request) {
        $filtros = array("NMPRODUTO", "CDARVPROD");
        
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = array(
                "columnName" => $key,
                "operator"=> !in_array($key, $filtros) ? "=" : "LIKE_I",
                "value"=> $value
            );
        }

        $filterCriteria = new FilterCriteria(
                $this->dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
        );
        $newRequest = new DTO\Request\Filter(
            $filterCriteria,
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        
        return $newRequest;
    }
    
    private function gerarParametros($nome, $conditions) {
        $params = array();
        
        switch ($nome) {
            case 'atualizarControleEstoque':
                $params = array(
                    ":CDFILIALENTR" => $conditions["CDFILIAL"], 
                    ":NRLANESTENTR" => $conditions["NRLANESTENTR"],
                    ":CDFILIALSAID" => $conditions["CDFILIAL"],
                    ":NRLANESTSAID" => $conditions["NRLANESTSAID"],
                    ":NRCTRLMOVEST" => $conditions["NRCTRLMOVEST"],
                    ":CDALMOXENTR"  => empty($conditions["CDALMOXENTR"]) ? " " : $conditions["CDALMOXENTR"],
                    ":CDALMOXSAID"  => empty($conditions["CDALMOXSAID"]) ? " " : $conditions["CDALMOXSAID"],
                    ":CDLCESTENTR"  => empty($conditions["CDLCESTENTR"]) ? " " : $conditions["CDLCESTENTR"],
                    ":CDLCESTSAID"  => empty($conditions["CDLCESTSAID"]) ? " " : $conditions["CDLCESTSAID"],
                    ":DTMOVIESTQ"   => $conditions["DTMOVIESTQ"],
                    ":CDFILIAL"     => $conditions["CDFILIAL"]
                );
                break;
            case 'atualizarItemAcabado':
                $params = array(
                    ":CDFILIAL"          => $conditions["CDFILIAL"], 
                    ":CDTIPORETI"        => $conditions["CDTIPORETI"],
                    ":NRLANCESTQ"        => $conditions["NRLANCESTQ"],
                    ":CDPRODUTO"         => $conditions["CDPRODUTO"],
                    ":CDPRODMOVI"        => $conditions["CDPRODMOVI"],
                    ":VRUNILANCTO"       => gettype($conditions['VRUNILANCTO']) == 'string' ? $this->utilService->moneyToDouble($conditions["VRUNILANCTO"]) : $conditions["VRUNILANCTO"],
                    ":VRTOTLANCTO"       => gettype($conditions['VRTOTLANCTO']) == 'string' ? $this->utilService->moneyToDouble($conditions["VRTOTLANCTO"]) : $conditions["VRTOTLANCTO"],
                    ":QTTOTLANCTO"       => gettype($conditions['QTTOTLANCTO']) == 'string' ? $this->utilService->moneyToDouble($conditions["QTTOTLANCTO"]) : $conditions["QTTOTLANCTO"],
                    ":QTLANCTOEST"       => gettype($conditions['QTLANCTOEST']) == 'string' ? $this->utilService->moneyToDouble($conditions["QTLANCTOEST"]) : $conditions["QTLANCTOEST"],
                    ":IDTIPOMOVI"        => $conditions["IDTIPOMOVI"],
                    ":DTLANCMOVI"        => $conditions["DTLANCMOVI"],
                    ":IDORDEMCLC"        => $conditions["IDORDEMCLC"],
                    ":VRLANCTOBRUT"      => $conditions["VRLANCTOBRUT"],
                	":VRLANCTOEST"       => $conditions["VRLANCTOEST"],
                	":CDFILIMOVI"        => $conditions["CDFILIMOVI"],
                	":CDALMOXARIFE"      => $conditions["CDALMOXARIFE"],
                    ":CDLOCALESTOQ"      => empty($conditions["NRLOTEESTQ"]) ? " " : $conditions["CDLOCALESTOQ"],
                    ":NRLOTEESTQ"        => empty($conditions["NRLOTEESTQ"]) ? " " : $conditions["NRLOTEESTQ"],
                    ":NRSUBLOTE"         => empty($conditions["NRSUBLOTE"]) ? " " : $conditions["NRSUBLOTE"],
	                ":NRSEQUITEM"        => $conditions["NRSEQUITEM"]
                );
                break;
            case 'inserirItemAcabado':
                $params = array(
                    ":CDFILIAL"      => $conditions["CDFILIAL"],
                    ":NRLANCESTQ"    => $conditions["NRLANCESTQ"],
                    ":NRSEQUITEM"    => $conditions["NRSEQUITEM"],
                    ":CDPRODUTO"     => $conditions["CDPRODUTO"],
                    ":CDPRODMOVI"    => $conditions["CDPRODUTO"],
                    ":VRUNILANCTO"   => gettype($conditions['VRUNILANCTO']) == 'string' ? $this->utilService->moneyToDouble($conditions["VRUNILANCTO"]) : $conditions["VRUNILANCTO"],
                    ":VRTOTLANCTO"   => gettype($conditions['VRTOTLANCTO']) == 'string' ? $this->utilService->moneyToDouble($conditions["VRTOTLANCTO"]) : $conditions["VRTOTLANCTO"],
                    ":QTTOTLANCTO"   => gettype($conditions['QTTOTLANCTO']) == 'string' ? $this->utilService->moneyToDouble($conditions["QTTOTLANCTO"]) : $conditions["QTTOTLANCTO"],
                    ":QTLANCTOEST"   => gettype($conditions['QTLANCTOEST']) == 'string' ? $this->utilService->moneyToDouble($conditions["QTLANCTOEST"]) : $conditions["QTLANCTOEST"],
                    ":IDTIPOMOVI"    => $conditions["IDTIPOMOVI"],
                    ":DTLANCMOVI"    => $conditions["DTLANCMOVI"],
                    ":IDORDEMCLC"    => $conditions["IDORDEMCLC"],
                    ":VRLANCTOBRUT"  => $conditions["VRLANCTOBRUT"],
                    ":VRLANCTOEST"   => $conditions["VRLANCTOEST"],
                    ":CDFILIMOVI"    => $conditions["CDFILIAL"],
                    ":CDALMOXARIFE"  => $conditions["CDALMOXENTR"],
                    ":CDLOCALESTOQ"  => empty($conditions["NRLOTEESTQ"]) ? " " : $conditions["CDLOCALESTOQ"],
                    ":NRLOTEESTQ"    => empty($conditions["NRLOTEESTQ"]) ? " " : $conditions["NRLOTEESTQ"],
                    ":NRSUBLOTE"     => empty($conditions["NRSUBLOTE"]) ? " " : $conditions["NRSUBLOTE"],
                    ":IDEVOLESTOQ"   => $conditions["IDEVOLESTOQ"],
                    ":CDTIPORETI"    => $conditions["CDTIPORETI"]
                );
                break;
            case 'lancarEntradaSaida':
                $dataHoje = date ('d/m/y');
                $params = array(
                    ":CDFILIAL"   => $conditions["CDFILIAL"],
                    ":NRLANCESTQ" => $conditions["NRLANCESTQ"],
                    ":DTLANCMOVI" => $dataHoje,
                    ":DTLANCESTQ" => $conditions["DTMOVIESTQ"],
                    ":CDFILIMOVI" => $conditions["CDFILIAL"],
                    ":CDOPERADOR" => $conditions["CDOPERADOR"],
                    ":DSLANCESTQ" => "Retirada para Produto Acabado: ".$conditions["NRLANCESTQ"]
                );
                break;
            case 'lancarEntradaSaidaItem':
                $dataHoje = date ('d/m/y');
                $params = array(
                    ":CDFILIAL"     => $conditions["CDFILIAL"],
                    ":CDALMOXARIFE" => empty($conditions["CDALMOXARIFE"]) ? " " : $conditions["CDALMOXARIFE"],
                    ":CDLOCALESTOQ" => empty($conditions["CDLOCALESTOQ"]) ? " " : $conditions["CDLOCALESTOQ"],
                    ":CDPRODUTO"    => $conditions["CDPRODUTO"]
                );
                break;
            case 'lancarEntradaSaidaItem2':
                $dataHoje = date ('d/m/y');
                
                $VRUNILANCTO = gettype($conditions['VRUNILANCTO']) == 'string' ? $this->utilService->moneyToDouble($conditions["VRUNILANCTO"]) : $conditions["VRUNILANCTO"];              
                $VRTOTLANCTO = gettype($conditions['VRTOTLANCTO']) == 'string' ? $this->utilService->moneyToDouble($conditions["VRTOTLANCTO"]) : $conditions["VRTOTLANCTO"];
                $QTTOTLANCTO = gettype($conditions['QTTOTLANCTO']) == 'string' ? $this->utilService->moneyToDouble($conditions["QTTOTLANCTO"]) : $conditions["QTTOTLANCTO"];
                
                $params = array(
                    ":CDFILIAL"     => $conditions["CDFILIAL"],
                    ":NRLANCESTQ"   => $conditions["NRLANCESTQ"],
                    ":NRSEQUITEM"   => $conditions["NRSEQUITEM"],
                    ":CDPRODUTO"    => $conditions["CDPRODUTO"],
                    ":CDPRODMOVI"   => $conditions["CDPRODUTO"],
                    ":VRUNILANCTO"  => $VRUNILANCTO,
                    ":VRTOTLANCTO"  => $VRTOTLANCTO, 
                    ":QTTOTLANCTO"  => $QTTOTLANCTO, 
                    ":QTLANCTOEST"  => ($QTTOTLANCTO  * $conditions["VRFATOCONV"]),
                    ":DTLANCMOVI"   => $dataHoje,
                    ":VRLANCTOBRUT" => gettype($conditions['VRLANCTOBRUT']) == 'string' ? $this->utilService->moneyToDouble($conditions["VRLANCTOBRUT"]) : $conditions["VRLANCTOBRUT"], 
                    ":VRLANCTOEST"  => gettype($conditions['VRLANCTOEST']) == 'string' ? $this->utilService->moneyToDouble($conditions["VRLANCTOEST"]) : $conditions["VRLANCTOEST"], 
                    ":CDFILIMOVI"   => $conditions["CDFILIAL"],
                    ":CDALMOXARIFE" => empty($conditions["CDALMOXARIFE"]) ? " " : $conditions["CDALMOXARIFE"],
                    ":CDLOCALESTOQ" => empty($conditions["CDLOCALESTOQ"]) ? " " : $conditions["CDLOCALESTOQ"], 
                    ":NRLOTEESTQ"   => empty($conditions["NRLOTEESTQ"]) ? " " : $conditions["NRLOTEESTQ"], 
                    ":NRSUBLOTE"    => empty($conditions["NRSUBLOTE"]) ? " " : $conditions["NRSUBLOTE"], 
                    ":IDEVOLESTOQ"  => $conditions["IDEVOLESTOQ"],
                    ":CDTIPORETI"   => $conditions["CDTIPORETI"]
                );
                break;
            case 'adqInsertLancto':
                $params = array(
                   ":CDFILIAL"   => $conditions["CDFILIAL"],
                   ":NRLANCESTQ" => $this->stNrLancSaid,
                   ":IDTPLANCTO" => '4',
                   ":DTLANCESTQ" => $conditions["DTMOVIESTQ"],
                   ":DSLANCESTQ" => self::msgRet.$this->stNrLancSaid,
                   ":CDFILIMOVI" => $conditions["CDFILIAL"],
                   ":CDOPERADOR" => $conditions["CDOPERADOR"]
                );
                break;
            case 'adqExisteCtrl':
                if (empty($conditions["NRCTRLMOVEST"])) $NRCTRLMOVEST = null;
                else $NRCTRLMOVEST = $conditions["NRCTRLMOVEST"];
                $params = array(
                    ":CDFILIAL"     => $conditions["CDFILIAL"],
                    ":NRCTRLMOVEST"   => $NRCTRLMOVEST
                );
                break;
            case 'adqProdFili':
                $params = array(
                    ":CDFILIAL"    => $conditions["CDFILIAL"],
                    ":CDARVPROD"   => $conditions["CDARVPROD"]
                );
                break;
            case 'adqUpdateLanctoEstoq':
                $params = array(
                    ":CDFILIAL"     => $conditions["CDFILIAL"],
                    ":NRLANCESTQ"   => $conditions["NRLANESTSAID"],
                    ":DTLANCESTQ"   => $conditions["DTMOVIESTQ"]
                );
                break;
            case 'adqUpdateDtLancMovi':
                $params = array(
                    ":CDFILIAL"     => $conditions["CDFILIAL"],
                    ":NRLANCESTQ"   => $conditions["NRLANESTSAID"],
                    ":DTLANCMOVI"   => $conditions["DTMOVIESTQ"]
                );
                break;
            case 'adqLote':
                $params = array(
                    ":CDFILIAL"     => $conditions["CDFILIAL"],
                    ":CDPRODUTO"    => $conditions["CDPRODUTO"],
                    ":CDALMOXARIFE" => $conditions["CDALMOXARIFE"],
                    ":CDLOCALESTOQ" => $conditions["CDLOCALESTOQ"]
                );
                break;
            case 'adqParamFilial':
                $params = array(
                    ":CDFILIAL"     => $conditions["CDFILIAL"]
                );
                break;
        }
        
        return $params;
    }
    
    public function addQueryNoLog ($query) {
        if (self::LOG_QUERY) {
            $this->addLog(self::qurExe.$query);
        }
    }
    
    private function addLog($string) {
        if (self::SALVA_LOG) {
            $this->logTela[] = $string;
        }
    }
    
    const INSERT_RELACIONAMENTO = "
	    INSERT INTO CTRLMOVIESTQ
          (CDFILIAL, NRCTRLMOVEST, CDFILIALENTR, NRLANESTENTR, CDFILIALSAID, 
           NRLANESTSAID,  CDALMOXENTR, CDALMOXSAID, CDLCESTENTR, CDLCESTSAID, DTMOVIESTQ)
        VALUES
          (:CDFILIAL, :NRCTRLMOVEST, :CDFILIALENTR, :NRLANESTENTR, :CDFILIALSAID, 
           :NRLANESTSAID, :CDALMOXENTR, :CDALMOXSAID, :CDLCESTENTR, :CDLCESTSAID,  :DTMOVIESTQ)
	";
    
    
    const INSERT_LANCEMENTO_ENTRADA_ITEM2 = "
        INSERT INTO ITLANCTOEST
          (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, CDPRODMOVI, VRUNILANCTO, 
           VRTOTLANCTO, QTTOTLANCTO, QTLANCTOEST, IDTIPOMOVI, DTLANCMOVI, IDORDEMCLC,
           VRLANCTOBRUT, VRLANCTOEST, CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ,
           NRLOTEESTQ, NRSUBLOTE, IDEVOLESTOQ, CDTIPORETI)
        VALUES
          (:CDFILIAL, :NRLANCESTQ, :NRSEQUITEM, :CDPRODUTO, :CDPRODMOVI, :VRUNILANCTO,
           :VRTOTLANCTO, :QTTOTLANCTO, :QTLANCTOEST, '2', :DTLANCMOVI,
           '2', :VRLANCTOBRUT, :VRLANCTOEST, :CDFILIMOVI, :CDALMOXARIFE, :CDLOCALESTOQ,
           :NRLOTEESTQ, :NRSUBLOTE, :IDEVOLESTOQ, :CDTIPORETI)";
    
    const INSERT_LANCEMENTO_ENTRADA_ITEM = "
	    INSERT INTO PRODLCESTOQ
        (CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO)
        (SELECT :CDFILIAL, :CDALMOXARIFE, :CDLOCALESTOQ, :CDPRODUTO
         FROM PRODFILI
         WHERE CDFILIAL  = :CDFILIAL  AND
               CDPRODUTO = :CDPRODUTO AND
               NVL(IDCNTRESTOQ, 'S') <> 'N'
         MINUS
         SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO
         FROM PRODLCESTOQ
         WHERE CDFILIAL     = :CDFILIAL     AND
               CDALMOXARIFE = :CDALMOXARIFE AND
               CDLOCALESTOQ = :CDLOCALESTOQ AND
               CDPRODUTO    = :CDPRODUTO)";
               
    const INSERT_LANCAMENTO_ENTRADA = "
	INSERT INTO LANCTOESTOQ
      (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, CDFILIMOVI, CDOPERADOR)
    VALUES (:CDFILIAL, :NRLANCESTQ, '2' , :DTLANCESTQ, :DSLANCESTQ, :CDFILIMOVI, :CDOPERADOR)";
    
	const INSERT_LANCAMENTO_SAIDA_ITEM = "
	    INSERT INTO PRODLCESTOQ
            (CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO)
        (SELECT :CDFILIAL, :CDALMOXARIFE, :CDLOCALESTOQ, :CDPRODUTO
         FROM PRODFILI
         WHERE CDFILIAL  = :CDFILIAL  AND
               CDPRODUTO = :CDPRODUTO AND
               NVL(IDCNTRESTOQ, 'S') <> 'N'
         MINUS
         SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO
         FROM PRODLCESTOQ
         WHERE CDFILIAL     = :CDFILIAL     AND
               CDALMOXARIFE = :CDALMOXARIFE AND
               CDLOCALESTOQ = :CDLOCALESTOQ AND
               CDPRODUTO    = :CDPRODUTO)";
	
	const INSERT_LANCAMENTO_SAIDA_ITEM2 = "
	    INSERT INTO ITLANCTOEST
          (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, CDPRODMOVI, VRUNILANCTO, 
           VRTOTLANCTO, QTTOTLANCTO, QTLANCTOEST, IDTIPOMOVI, DTLANCMOVI, IDORDEMCLC,
           VRLANCTOBRUT, VRLANCTOEST, CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ,
           NRLOTEESTQ, NRSUBLOTE, IDEVOLESTOQ, CDTIPORETI)
        VALUES
          (:CDFILIAL, :NRLANCESTQ, :NRSEQUITEM, :CDPRODUTO, :CDPRODMOVI, :VRUNILANCTO,
           :VRTOTLANCTO, :QTTOTLANCTO, :QTLANCTOEST, '3', :DTLANCMOVI,
           '9' , :VRLANCTOBRUT, :VRLANCTOEST, :CDFILIMOVI, :CDALMOXARIFE, :CDLOCALESTOQ,
           :NRLOTEESTQ, :NRSUBLOTE, :IDEVOLESTOQ, :CDTIPORETI)";
    
    const INSERT_LANCAMENTO_SAIDA = "
	    INSERT INTO LANCTOESTOQ
            (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, CDFILIMOVI, CDOPERADOR)
        VALUES
            (:CDFILIAL, :NRLANCESTQ, '4', :DTLANCESTQ, :DSLANCESTQ, :CDFILIMOVI, :CDOPERADOR)";
	
    const INSERT_ITEM_ACABADO = "
    INSERT INTO ITLANCTOEST
      (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, CDPRODMOVI, VRUNILANCTO, 
       VRTOTLANCTO, QTTOTLANCTO, QTLANCTOEST, IDTIPOMOVI, DTLANCMOVI, IDORDEMCLC,
       VRLANCTOBRUT, VRLANCTOEST, CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ,
       NRLOTEESTQ, NRSUBLOTE, IDEVOLESTOQ, CDTIPORETI)
    VALUES
      (:CDFILIAL, :NRLANCESTQ, :NRSEQUITEM, :CDPRODUTO, :CDPRODMOVI, :VRUNILANCTO,
       :VRTOTLANCTO, :QTTOTLANCTO, :QTLANCTOEST, :IDTIPOMOVI, :DTLANCMOVI,
       :IDORDEMCLC, :VRLANCTOBRUT, :VRLANCTOEST, :CDFILIMOVI, :CDALMOXARIFE, :CDLOCALESTOQ,
       :NRLOTEESTQ, :NRSUBLOTE, :IDEVOLESTOQ, :CDTIPORETI)";
	
	// Parâmetros: CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, CDPRODMOVI, VRUNILANCTO, 
	// VRTOTLANCTO, QTTOTLANCTO, QTLANCTOEST, IDTIPOMOVI, DTLANCMOVI, IDORDEMCLC, VRLANCTOBRUT, 
	// VRLANCTOEST, CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE,
	// NRLANCESTQ, NRSEQUITEM, CDTIPORETI
	const UPDATE_ITEM_ACABADO = "
	UPDATE ITLANCTOEST
    SET
      CDFILIAL     = :CDFILIAL,
      NRLANCESTQ   = :NRLANCESTQ,
      NRSEQUITEM   = :NRSEQUITEM,
      CDPRODUTO    = :CDPRODUTO,
      CDPRODMOVI   = :CDPRODMOVI,
      VRUNILANCTO  = :VRUNILANCTO,
      VRTOTLANCTO  = :VRTOTLANCTO,
      QTTOTLANCTO  = :QTTOTLANCTO,
      QTLANCTOEST  = :QTLANCTOEST,
      IDTIPOMOVI   = :IDTIPOMOVI,
      DTLANCMOVI   = :DTLANCMOVI,
      IDORDEMCLC   = :IDORDEMCLC,
      VRLANCTOBRUT = :VRLANCTOBRUT,
      VRLANCTOEST  = :VRLANCTOEST,
      CDFILIMOVI   = :CDFILIMOVI,
      CDALMOXARIFE = :CDALMOXARIFE,
      CDLOCALESTOQ = :CDLOCALESTOQ,
      NRLOTEESTQ   = :NRLOTEESTQ,
      NRSUBLOTE    = :NRSUBLOTE,
      CDTIPORETI   = :CDTIPORETI
    WHERE CDFILIAL = :CDFILIAL
      AND NRLANCESTQ = :NRLANCESTQ
      AND NRSEQUITEM = :NRSEQUITEM";
	
	const UPDATE_CTRL_ESTOQUE = "
	    UPDATE CTRLMOVIESTQ
        SET
          CDFILIALENTR = :CDFILIALENTR,
          NRLANESTENTR = :NRLANESTENTR,
          CDFILIALSAID = :CDFILIALSAID,
          NRLANESTSAID = :NRLANESTSAID,
          CDALMOXENTR = :CDALMOXENTR,
          CDALMOXSAID = :CDALMOXSAID,
          CDLCESTENTR = :CDLCESTENTR,
          CDLCESTSAID = :CDLCESTSAID,
          DTMOVIESTQ = :DTMOVIESTQ
        WHERE CDFILIAL = :CDFILIAL
          AND NRCTRLMOVEST = :NRCTRLMOVEST";
          
    const PARAMETROS_FILIAL = "SELECT IDCTRLESTQ, IDUTILALMOX, IDUTILLCESTQ, IDUTILLOTE FROM PARAMFILIAL WHERE CDFILIAL = :CDFILIAL";

    const PARAM_FILIAL = "SELECT CDFILIAL, IDESTONLINE, IDPERMESTNEG, IDORDIMPLANT, IDCTRLESTQ, IDUTILLCESTQ, 
                                 IDUTILLOTE, IDUTCODBARFI, IDUTILALMOX, NVL(CDTIPCUSTEST, '3') CDTIPCUSTEST, IDLANCDTMENFIL, 
                                 NRDIASSAIDFIL, IDLANCDTMAIFIL, NRDIASSAIDPDFIL, IDPERMVISPOSEST
                          FROM PARAMFILIAL 
                          WHERE CDFILIAL = :CDFILIAL";
                          
    const ADQEXISTECTRL = "SELECT C.CDFILIAL, C.NRCTRLMOVEST
                           FROM CTRLMOVIESTQ C
                           WHERE C.CDFILIAL     = :CDFILIAL
                             AND C.NRCTRLMOVEST = :NRCTRLMOVEST";
                             
    const ADQINSERTLANCTO = "INSERT INTO LANCTOESTOQ
                                (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ, CDFILIMOVI, CDOPERADOR)
                            VALUES 
                                (:CDFILIAL, :NRLANCESTQ, :IDTPLANCTO, :DTLANCESTQ, :DSLANCESTQ, :CDFILIMOVI, :CDOPERADOR)";
                                
    const ADQPRODFILI = "SELECT CDFILIAL, F.CDPRODUTO, NVL(IDCNTRESTOQ, 'S') IDCNTRESTOQ,
                                NVL(IDPRODATIV, 'A') IDPRODATIV, CDARVPROD, CDPRODESTO
                        FROM PRODFILI F, PRODUTO P
                        WHERE (F.CDFILIAL = :CDFILIAL)
                          AND (P.CDARVPROD = :CDARVPROD)
                          AND (F.CDPRODUTO = P.CDPRODUTO)";
                                 
    const ADQUPDATELANCTOESTOQ = "UPDATE LANCTOESTOQ 
                                    SET DTLANCESTQ = :DTLANCESTQ
                                  WHERE CDFILIAL   = :CDFILIAL 
                                    AND NRLANCESTQ = :NRLANCESTQ";
                                    
    const ADQUPDATEDTLANCMOVI = "UPDATE ITLANCTOEST
                                    SET DTLANCMOVI = :DTLANCMOVI
                                 WHERE CDFILIAL    = :CDFILIAL 
                                   AND NRLANCESTQ  = :NRLANCESTQ";
                                   
    const ADQLOTE = "SELECT NRLOTEESTQ, NRSUBLOTE, QTTOTALESTOQ, VRTOTALESTOQ
                     FROM POSESTATU
                     WHERE CDFILIAL     = :CDFILIAL     
                       AND CDALMOXARIFE = :CDALMOXARIFE 
                       AND CDLOCALESTOQ = :CDLOCALESTOQ
                       AND CDPRODUTO    = :CDPRODUTO
                       AND QTTOTALESTOQ > 0 
                     ORDER BY NRLOTEESTQ, NRSUBLOTE";
                     
    const ADQPARAMFILIAL = "SELECT IDPERMESTNEG, IDBLQESTFMES, NRDIABLOQEST,
                                   IDORDIMPLANT, IDORDAJUINVE, IDCTRLESTQ,
                                   NRDIAFECHMES, IDINVROTOK,   CDFILIAL,
                                   CDALMOXARIFE, CDLOCALESTOQ, IDUTCACALMOX,
                                   IDUTILALMOX,  IDUTILLCESTQ, IDUTILLOTE,
                                   DTPROCESSA,   NVL(CDTIPCUSTEST, '3') CDTIPCUSTEST
                            FROM PARAMFILIAL
                            WHERE CDFILIAL = :CDFILIAL";
                            
    const ADQPRODFILIEST = "SELECT PF.CDFILIAL, PF.CDPRODUTO, PF.IDCNTRESTOQ, PF.IDUTCTRLLOTE,
                                   PF.CDALMOXARIFE, PF.CDLOCALESTOQ, F.IDUTILLOTE
                            FROM
                                PRODFILI PF, PRODUTO P, PARAMFILIAL F
                            WHERE (PF.CDFILIAL  = :CDFILIAL)  
                              AND (P.CDPRODUTO  = :CDPRODUTO) 
                              AND (PF.CDPRODUTO = P.CDPRODESTO) 
                              AND (PF.CDFILIAL  = F.CDFILIAL)";
                              
    const ACMINSERTPRODLCESTOQUE = "
        INSERT INTO PRODLCESTOQ
            (CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO) 
        (SELECT :CDFILIAL, :CDALMOXARIFE, :CDLOCALESTOQ, :CDPRODUTO
         FROM PRODFILI
         WHERE CDFILIAL  = :CDFILIAL
         AND CDPRODUTO = :CDPRODUTO 
         AND NVL(IDCNTRESTOQ, 'S') <> 'N'
         MINUS
         SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO
         FROM PRODLCESTOQ
         WHERE CDFILIAL = :CDFILIAL     
         AND CDALMOXARIFE = :CDALMOXARIFE 
         AND CDLOCALESTOQ = :CDLOCALESTOQ 
         AND CDPRODUTO    = :CDPRODUTO)";
         
    const ADQVERIFLOTE = "SELECT CDFILIAL
                          FROM LOTE
                          WHERE (:CDFILIAL   =  CDFILIAL)   
                            AND (:NRLOTEESTQ =  NRLOTEESTQ) 
                            AND (:NRSUBLOTE  =  NRSUBLOTE)
                            AND (:CDPRODUTO  =  CDPRODUTO)";
                            
    const ACMINSERTLOTE = "
        INSERT INTO LOTE
        (CDFILIAL, NRLOTEESTQ, NRSUBLOTE, CDPRODUTO, IDSTATLOTE, DTENTRLOTE)
        (SELECT :CDFILIAL CDFILIAL, :NRLOTEESTQ NRLOTEESTQ, :NRSUBLOTE NRSUBLOTE, :CDPRODUTO CDPRODUTO,
                'L' IDSTATLOTE, SYSDATE DTENTRLOTE
         FROM PARAMGERAL
         MINUS 
         SELECT CDFILIAL, NRLOTEESTQ, NRSUBLOTE, :CDPRODUTO CDPRODUTO, 'L', SYSDATE
         FROM LOTE
         WHERE :CDFILIAL   =  CDFILIAL
           AND :NRLOTEESTQ =  NRLOTEESTQ
           AND :NRSUBLOTE  =  NRSUBLOTE
           AND :CDPRODUTO  =  CDPRODUTO)";
           
    const ACMDELETEMOVILOTEORIG = "
        DELETE FROM MOVILOTE
        WHERE (NRLANCTOORIG = :NRLANCTOORIG)";
        
    const ACMDELETEMOVILOTE = "
        DELETE FROM MOVILOTE
        WHERE (NRMOVILOTE = :NRMOVILOTE)";
        
    const ACMDELETEMOVILOTEPROD = "
        DELETE FROM MOVILOTE 
        WHERE (NRMOVILOTE = :NRMOVILOTE) 
          AND (CDPRODUTO  = :CDPRODUTO)";
          
    const ADQMOVILOTEORIG  = "
        SELECT SUBSTR(' '|| PR.CDPRODUTO ||' - ' ||PR.NMPRODUTO, 1, 60) AS DSPRODUTO,
               SUBSTR(' '||F.CDFILIAL  ||' - '||F.NMFILIAL,1,30)   AS DSFILIAL
        FROM MOVILOTE M, PRODUTO PR, FILIAL F
        WHERE (NRLANCTOORIG = :NRLANCTOORIG) 
          AND (M.CDPRODUTO  = PR.CDPRODUTO)    
          AND (M.CDFILIAL   = F.CDFILIAL)
        ORDER BY M.CDPRODUTO";
        
    const ADQPARAMFILI = "
        SELECT P.CDFILIAL
        FROM PARAMFILIAL P, MOVILOTE M
        WHERE ((NRLANCTOORIG = :NRLANCTOORIG) OR (M.NRMOVILOTE  = :NRLANCTOORIG))   
          AND (M.CDFILIAL   = P.CDFILIAL)      
          AND (IDPERMESTNEG = 'N' OR IDPERMESTNEG IS NULL)";
          
    const ADQLANCTOESTOQ = "
        SELECT CDFILIMOVI
        FROM LANCTOESTOQ
        WHERE CDFILIAL = :CDFILIAL   
          AND NRLANCESTQ = :NRLANCESTQ";
        
    const ADQMOVILOTE = "
        SELECT (P.CDARVPROD||' '||P.NMPRODUTO) AS DSPRODUTO,
               (M.CDFILIAL||' '||F.NMFILIAL)   AS DSFILIAL, P.CDPRODUTO
        FROM MOVILOTE M, PRODUTO P, FILIAL F
        WHERE (NRMOVILOTE  = :NRMOVILOTE)   
          AND (M.CDFILIAL  = F.CDFILIAL)      
          AND (M.CDPRODUTO = P.CDPRODUTO)";
          
    const ADQMOVILOTEPROD = "
    SELECT CDPRODUTO
      FROM MOVILOTE
      WHERE (NRMOVILOTE = :NRMOVILOTE) AND
            (CDPRODUTO  = :CDPRODUTO)";
            
    const DATAPROC = "SELECT TO_CHAR(NVL(DTPROCESSA, SYSDATE), 'DD/MM/YYYY') DTPROCESSA FROM PARAMFILIAL WHERE CDFILIAL = :CDFILIAL";

    const ACMDELCTRLMOVIESTQ = "DELETE FROM CTRLMOVIESTQ
                                 WHERE CDFILIAL = :CDFILIAL 
                                   AND NRCTRLMOVEST = :NRCTRLMOVEST";
    
    const ACMDELITLANCTOEST = "DELETE FROM ITLANCTOEST
              WHERE CDFILIAL   = :CDFILIAL
                AND NRLANCESTQ = :NRLANCESTQ";
                
    const ACMDELLANCTOESTQ = "DELETE FROM LANCTOESTOQ
                               WHERE CDFILIAL   = :CDFILIAL 
                                 AND NRLANCESTQ = :NRLANCESTQ";
                                 
    const ADQMAXDTPOSEST = "
        SELECT TO_CHAR(NVL(MAX(DTPOSIESTQ), SYSDATE), 'DD/MM/YYYY') DTPOSIESTQ
        FROM POSESTDIA
        WHERE (CDFILIAL = :CDFILIAL)
          AND (CDALMOXARIFE = :CDALMOXARIFE)
          AND (CDLOCALESTOQ = :CDLOCALESTOQ)
          AND (NRLOTEESTQ = :NRLOTEESTQ)
          AND (NRSUBLOTE = :NRSUBLOTE)
          AND (CDPRODUTO = :CDPRODESTO)
          AND (DTPOSIESTQ <= :DTPOSICAO)"; 
          
    const ADQPOSESTLANC = "
        SELECT P.CDPRODUTO,  P.NMPRODUTO,  P.SGUNIDADE, IDCNTRESTOQ,
               P.VRFATOCONV, (D.QTESTOQDIA/P.VRFATOCONV) + NVL((QTLANCTOEST/P.VRFATOCONV),0) AS QTESTOQDIA,
               P.CDPRODESTO,
               NVL(CUSTO_PRODUTO_LCEST(:CDFILIAL, D.CDPRODUTO, TO_NUMBER(NVL(CDTIPCUSTEST, '3')), :DTCUSTO, SYSDATE,
               D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOPROD,
              NVL(CUSTO_PRODUTO_LCEST(:CDFILIAL, D.CDPRODUTO, 4, :DTCUSTO, SYSDATE,
               D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOBRUT
        FROM PRODUTO P, POSESTDIA D, PRODFILI F, PARAMFILIAL PF,
             (SELECT QTLANCTOEST, CDPRODUTO
              FROM ITLANCTOEST
              WHERE CDPRODMOVI   = :CDPRODUTO   AND
                    CDFILIMOVI  = :CDFILIAL    AND
                    CDFILIAL    = :CDFILIPROC  AND
                    NRLANCESTQ  = :NRLANCESTQ) I
        
        WHERE (P.CDPRODUTO    = :CDPRODUTO)    AND
              (D.CDFILIAL     = :CDFILIAL)     AND
              (D.CDALMOXARIFE = :CDALMOXARIFE) AND
              (D.CDLOCALESTOQ = :CDLOCALESTOQ) AND
              (D.NRLOTEESTQ   = :NRLOTEESTQ)   AND
              (D.NRSUBLOTE    = :NRSUBLOTE)    AND
              (D.CDPRODUTO    = P.CDPRODESTO)    AND
              (D.CDPRODUTO    = I.CDPRODUTO(+))  AND
              (P.CDPRODESTO   = F.CDPRODUTO)     AND
              (F.CDFILIAL     = D.CDFILIAL)      AND
              (PF.CDFILIAL    = D.CDFILIAL)     AND
              (DTPOSIESTQ     = :DTPOSICAO)";
    
    const PRODUTOS_COMP = "
        SELECT P.CDPRODUTO, P.CDARVPROD, R.QTSUBPRODROT AS QTITEMCOMPOS,
          R.NRHRFABPROD, R.NRSEQPROD AS NRSEQUE, R.IDIMPPRODROT, P.NMPRODUTO,
          P.CDPRODESTO, P.SGUNIDADE, P.VRFATOCONV, PF.CDFILIAL,
          PF.IDCNTRESTOQ, R.NRORG, NVL(NVL(PF.CDALMOXARIFE, PR.CDALMOXARIFE), ' ') AS CDALMOXARIFE, NVL(NVL(PF.CDLOCALESTOQ, PR.CDLOCALESTOQ), ' ') AS CDLOCALESTOQ
        FROM ROTFABRICA R,
          PRODUTO P,
          PRODUTO O,
          (SELECT COUNT(*) AS NRCOMPPROD, CDPRODUTO FROM ROTFABRICA GROUP BY CDPRODUTO
          ) F,  
          PRODFILI PF,
          PARAMFILIAL PR
        WHERE (O.CDPRODUTO  = :CDPRODUTO)
        AND (O.CDPRODUTO    = R.CDPRODUTO)
        AND (R.CDSUBPRODUTO = P.CDPRODUTO)
        AND (F.CDPRODUTO(+) = R.CDSUBPRODUTO)
        AND (PF.CDFILIAL    = :CDFILIAL)
        AND (PF.CDPRODUTO   = P.CDPRODUTO)
        AND (PR.CDFILIAL    = :CDFILIAL)
        ORDER BY NRSEQUE";
        
    const PRODUTOS_COMP_L = "
        SELECT P.CDPRODUTO, P.CDARVPROD, R.QTSUBPRODLOC AS QTITEMCOMPOS,
          R.NRHRFABPRLOC, R.NRSEQPRODLOC AS NRSEQUE, R.IDIMPPROTLOC, P.NMPRODUTO,
          P.CDPRODESTO, P.SGUNIDADE, P.VRFATOCONV, PF.CDFILIAL,
          PF.IDCNTRESTOQ, R.NRORG, NVL(NVL(PF.CDALMOXARIFE, PR.CDALMOXARIFE), ' ') AS CDALMOXARIFE, NVL(NVL(PF.CDLOCALESTOQ, PR.CDLOCALESTOQ), ' ') AS CDLOCALESTOQ
        FROM ROTFABLOCAL R,
          PRODUTO P,
          PRODUTO O,
          (SELECT COUNT(*) AS NRCOMPPRODL, CDPRODUTO FROM ROTFABLOCAL GROUP BY CDPRODUTO
          ) F,  
          PRODFILI PF,
          PARAMFILIAL PR
        WHERE (O.CDPRODUTO  = :CDPRODUTO)
        AND (O.CDPRODUTO    = R.CDPRODUTO)
        AND (R.CDSUBPRODLOC = P.CDPRODUTO)
        AND (F.CDPRODUTO(+) = R.CDSUBPRODLOC)
        AND (PF.CDFILIAL    = :CDFILIAL)
        AND (PF.CDPRODUTO   = P.CDPRODUTO)
        AND (R.CDFILIAL     = :CDFILIAL)
        AND (PR.CDFILIAL    = :CDFILIAL)
        ORDER BY NRSEQUE";
        
    const PRODUTOS_ESTOQ = "    
        SELECT P.CDPRODUTO, P.CDARVPROD, 1 AS QTITEMCOMPOS,
          NULL AS NRHRFABPRLOC, NULL AS NRSEQUE, NULL AS IDIMPPROTLOC, P.NMPRODUTO,
          P.CDPRODESTO, P.SGUNIDADE, P.VRFATOCONV, PF.CDFILIAL,
          PF.IDCNTRESTOQ, P.NRORG, NVL(NVL(PF.CDALMOXARIFE, PR.CDALMOXARIFE), ' ') AS CDALMOXARIFE, NVL(NVL(PF.CDLOCALESTOQ, PR.CDLOCALESTOQ), ' ') AS CDLOCALESTOQ
        FROM PRODUTO P,
          PRODFILI PF,
          PARAMFILIAL PR
        WHERE (P.CDPRODUTO  = :CDPRODUTO)
        AND (PF.CDFILIAL    = :CDFILIAL)
        AND (PF.CDPRODUTO   = P.CDPRODUTO)
        AND (PR.CDFILIAL    = :CDFILIAL)";
        
}