<?php 

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;
use \Doctrine\ORM\EntityManager;


class RelProdutosMaisVendidos {
    
	protected $entityManager; 

	public function __construct(EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}


	private function setParameters($conditions) {
    	return array (
            ':DTINI' => $conditions['DTINI'], 
            ':DTFIN' => $conditions['DTFIN'], 
            ':CDFILIAL' => $conditions['CDFILIAL'], 
            ':TCDFILIAL' => $conditions['TCDFILIAL'], 
            ':CDLOJA' => $conditions['CDLOJA'], 
            ':TCDLOJA' => $conditions['TCDLOJA'], 
            ':CDCAIXA' => $conditions['CDCAIXA'], 
            ':TCDCAIXA' => $conditions['TCDCAIXA'], 
            ':NRORG' => $conditions['NRORG'], 
            ':TCDPRODUTO' => $conditions['TCDPRODUTO'],
            ':CDPRODLIST' => $conditions['CDPRODLIST'],
            ':CDARVPRODF' => $conditions['CDARVPRODF'],
            ':CDARVPRODI' => $conditions['CDARVPRODI'],
            ':PRODNIV' => $conditions['PRODNIV']
        );
    }
    
    public function getProdutosMaisVendidosPorQtde($conditions) {
        $params = $this->setParameters($conditions);
        return $this->entityManager->getConnection()->fetchAll(self::REL_PRODUTOS_MAIS_VENDIDOS_QTDE, $params);
    }
    
    public function getProdutosMaisVendidosPorValor($conditions) {
        $params = $this->setParameters($conditions);
        return $this->entityManager->getConnection()->fetchAll(self::REL_PRODUTOS_MAIS_VENDIDOS_VALOR, $params);
    }
    
    public function getProdutosMaisVendidosConsolid($conditions) {
        $params = $this->setParameters($conditions);
        return $this->entityManager->getConnection()->fetchAll(self::REL_PRODUTOS_MAIS_VENDIDOS_CONS, $params);
    }

  const REL_PRODUTOS_MAIS_VENDIDOS_VALOR ="
    SELECT (A.VRTOTVEND/SUM(A.VRTOTVEND) OVER (PARTITION BY A.CDFILIAL))*100 AS PERCENTUAL, A.CDFILIAL||' - '||A.NMFILIAL||' - Loja: '||A.CDLOJA||' - '||A.NMLOJA AS LABELFILIAL, A.* 
      FROM (
          SELECT I.CDPRODUTO, P.CDARVPROD||' - '||P.NMPRODUTO AS NMPRODUTO, I.CDFILIAL, FILIVENDCX.NMFILIAL, I.VRUNITVEND, SUM(I.QTPRODVEND) AS QTDE, 
                 SUM(I.QTPRODVEND * (I.VRUNITVEND + I.VRUNITVENDCL) - I.VRDESITVEND + I.VRACRITVEND) AS VRTOTVEND, FILIVENDCX.CDLOJA, FILIVENDCX.NMLOJA,
                 SUM(I.VRACRITVEND) AS ACR, SUM(I.VRDESITVEND) AS DESCO, SUM(I.QTPRODVEND*(I.VRUNITVEND + I.VRUNITVENDCL) - I.VRDESITVEND + I.VRACRITVEND)/SUM(I.QTPRODVEND) AS VRPRECMEDIO
            FROM ITEMVENDA I
            JOIN (SELECT FILICX.CDFILIAL, FILICX.CDCAIXA, FILICX.NMCAIXA, FILICX.NRORG, FILICX.NMFILIAL, FILICX.CDLOJA, V.NRSEQVENDA, V.IDSITUVENDA, L.NMLOJA
                    FROM VENDA V
                    JOIN (SELECT DISTINCT F.CDFILIAL, C.CDCAIXA, C.NMCAIXA, C.CDLOJA, F.NRORG, F.NMFILIAL
                            FROM (SELECT F.CDFILIAL, F.NRORG, F.NMFILIAL
                                    FROM FILIAL F
                                   WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))) F, CAIXA C
                           WHERE F.CDFILIAL = C.CDFILIAL
                             AND F.NRORG = C.NRORG
                             AND ((INSTR(:CDCAIXA, C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                             AND ((INSTR(:CDLOJA, F.CDFILIAL ||C.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
                             AND ((INSTR(:CDCAIXA, F.CDFILIAL||C.CDLOJA||C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                             AND F.NRORG = :NRORG) FILICX
                      ON V.CDFILIAL = FILICX.CDFILIAL
                     AND V.CDCAIXA = FILICX.CDCAIXA
                     AND V.CDLOJA = FILICX.CDLOJA
                     AND V.NRORG = FILICX.NRORG
                    JOIN LOJA L
                      ON V.CDLOJA = L.CDLOJA
                     AND V.CDFILIAL = L.CDFILIAL
                     AND V.NRORG = L.NRORG
                   WHERE ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                     AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                     AND V.NRORG = :NRORG
                     AND V.IDSITUVENDA = 'O' 
                   ORDER BY V.NRSEQVENDA
                 ) FILIVENDCX
              ON I.CDFILIAL   = FILIVENDCX.CDFILIAL
             AND I.CDCAIXA    = FILIVENDCX.CDCAIXA
             AND I.NRSEQVENDA = FILIVENDCX.NRSEQVENDA
             AND I.NRORG      = FILIVENDCX.NRORG
            JOIN PRODUTO P
              ON I.CDPRODUTO = P.CDPRODUTO
             AND I.NRORG = P.NRORG
           WHERE (((((INSTR(:CDPRODLIST, P.CDPRODUTO) > 0 ) OR (P.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF ))) OR (P.CDARVPROD LIKE :PRODNIV)) OR (:TCDPRODUTO = 'T'))
           GROUP BY I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, I.CDFILIAL, FILIVENDCX.NMFILIAL, I.VRUNITVEND, FILIVENDCX.CDLOJA, FILIVENDCX.NMLOJA
         ) A 
     ORDER BY A.CDFILIAL, A.QTDE DESC, A.VRTOTVEND DESC
  ";

  const REL_PRODUTOS_MAIS_VENDIDOS_QTDE ="
    SELECT (A.VRTOTVEND/SUM(A.VRTOTVEND) OVER (PARTITION BY A.CDFILIAL))*100 AS PERCENTUAL, A.CDFILIAL||' - '||A.NMFILIAL||' - Loja: '||A.CDLOJA||' - '||A.NMLOJA AS LABELFILIAL, A.* 
      FROM (
          SELECT I.CDPRODUTO, P.CDARVPROD||' - '||P.NMPRODUTO AS NMPRODUTO, I.CDFILIAL, FILIVENDCX.NMFILIAL, I.VRUNITVEND, SUM(I.QTPRODVEND) AS QTDE, 
                 SUM(I.QTPRODVEND * (I.VRUNITVEND + I.VRUNITVENDCL) - I.VRDESITVEND + I.VRACRITVEND) AS VRTOTVEND, FILIVENDCX.CDLOJA, FILIVENDCX.NMLOJA,
                 SUM(I.VRACRITVEND) AS ACR, SUM(I.VRDESITVEND) AS DESCO, SUM(I.QTPRODVEND*(I.VRUNITVEND + I.VRUNITVENDCL) - I.VRDESITVEND + I.VRACRITVEND)/SUM(I.QTPRODVEND) AS VRPRECMEDIO
            FROM ITEMVENDA I
            JOIN (SELECT FILICX.CDFILIAL, FILICX.CDCAIXA, FILICX.NMCAIXA, FILICX.NRORG, FILICX.NMFILIAL, FILICX.CDLOJA, V.NRSEQVENDA, V.IDSITUVENDA, L.NMLOJA
                    FROM VENDA V
                    JOIN (SELECT DISTINCT F.CDFILIAL, C.CDCAIXA, C.NMCAIXA, C.CDLOJA, F.NRORG, F.NMFILIAL
                            FROM (SELECT F.CDFILIAL, F.NRORG, F.NMFILIAL
                                    FROM FILIAL F
                                   WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))) F, CAIXA C
                           WHERE F.CDFILIAL = C.CDFILIAL
                             AND F.NRORG = C.NRORG
                             AND ((INSTR(:CDCAIXA, C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                             AND ((INSTR(:CDLOJA, F.CDFILIAL ||C.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
                             AND ((INSTR(:CDCAIXA, F.CDFILIAL||C.CDLOJA||C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                             AND F.NRORG = :NRORG) FILICX
                      ON V.CDFILIAL = FILICX.CDFILIAL
                     AND V.CDCAIXA = FILICX.CDCAIXA
                     AND V.CDLOJA = FILICX.CDLOJA
                     AND V.NRORG = FILICX.NRORG
                    JOIN LOJA L
                      ON V.CDLOJA = L.CDLOJA
                     AND V.CDFILIAL = L.CDFILIAL
                     AND V.NRORG = L.NRORG
                   WHERE ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                     AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                     AND V.NRORG = :NRORG
                     AND V.IDSITUVENDA = 'O' 
                   ORDER BY V.NRSEQVENDA
                 ) FILIVENDCX
              ON I.CDFILIAL   = FILIVENDCX.CDFILIAL
             AND I.CDCAIXA    = FILIVENDCX.CDCAIXA
             AND I.NRSEQVENDA = FILIVENDCX.NRSEQVENDA
             AND I.NRORG      = FILIVENDCX.NRORG
            JOIN PRODUTO P
              ON I.CDPRODUTO = P.CDPRODUTO
             AND I.NRORG = P.NRORG
           WHERE (((((INSTR(:CDPRODLIST, P.CDPRODUTO) > 0 ) OR (P.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF ))) OR (P.CDARVPROD LIKE :PRODNIV)) OR (:TCDPRODUTO = 'T'))
           GROUP BY I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, I.CDFILIAL, FILIVENDCX.NMFILIAL, I.VRUNITVEND, FILIVENDCX.CDLOJA, FILIVENDCX.NMLOJA
         ) A 
     ORDER BY A.CDFILIAL, A.QTDE DESC
  ";

  const REL_PRODUTOS_MAIS_VENDIDOS_CONS="
    SELECT (A.VRTOTVEND/SUM(A.VRTOTVEND) OVER (PARTITION BY A.CDFILIAL))*100 AS PERCENTUAL, A.CDFILIAL||' - '||A.NMFILIAL||' - Loja: '||A.CDLOJA||' - '||A.NMLOJA AS LABELFILIAL, A.* 
      FROM (
          SELECT I.CDPRODUTO, P.CDARVPROD, P.CDARVPROD||' - '||P.NMPRODUTO AS NMPRODUTO, I.CDFILIAL, FILIVENDCX.NMFILIAL, SUM(I.QTPRODVEND) AS QTDE, 
                 SUM(I.QTPRODVEND * (I.VRUNITVEND + I.VRUNITVENDCL) - I.VRDESITVEND + I.VRACRITVEND) AS VRTOTVEND, FILIVENDCX.CDLOJA, FILIVENDCX.NMLOJA,
                 SUM(I.VRACRITVEND) AS ACR, SUM(I.VRDESITVEND) AS DESCO, SUM(I.QTPRODVEND*(I.VRUNITVEND + I.VRUNITVENDCL) - I.VRDESITVEND + I.VRACRITVEND)/SUM(I.QTPRODVEND) AS VRPRECMEDIO
            FROM ITEMVENDA I
            JOIN (SELECT FILICX.CDFILIAL, FILICX.CDCAIXA, FILICX.NMCAIXA, FILICX.NRORG, FILICX.NMFILIAL, FILICX.CDLOJA, V.NRSEQVENDA, V.IDSITUVENDA, L.NMLOJA
                    FROM VENDA V
                    JOIN (SELECT DISTINCT F.CDFILIAL, C.CDCAIXA, C.NMCAIXA, C.CDLOJA, F.NRORG, F.NMFILIAL
                            FROM (SELECT F.CDFILIAL, F.NRORG, F.NMFILIAL
                                    FROM FILIAL F
                                   WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))) F, CAIXA C
                           WHERE F.CDFILIAL = C.CDFILIAL
                             AND F.NRORG = C.NRORG
                             AND ((INSTR(:CDCAIXA, C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                             AND ((INSTR(:CDLOJA, F.CDFILIAL ||C.CDLOJA) <> 0) OR (:TCDLOJA = 'T'))
                             AND ((INSTR(:CDCAIXA, F.CDFILIAL||C.CDLOJA||C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                             AND F.NRORG = :NRORG) FILICX
                      ON V.CDFILIAL = FILICX.CDFILIAL
                     AND V.CDCAIXA = FILICX.CDCAIXA
                     AND V.CDLOJA = FILICX.CDLOJA
                     AND V.NRORG = FILICX.NRORG
                    JOIN LOJA L
                      ON V.CDLOJA = L.CDLOJA
                     AND V.CDFILIAL = L.CDFILIAL
                     AND V.NRORG = L.NRORG
                   WHERE ((INSTR(:CDFILIAL, V.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                     AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                     AND V.NRORG = :NRORG
                     AND V.IDSITUVENDA = 'O' 
                   ORDER BY V.NRSEQVENDA
                 ) FILIVENDCX
              ON I.CDFILIAL   = FILIVENDCX.CDFILIAL
             AND I.CDCAIXA    = FILIVENDCX.CDCAIXA
             AND I.NRSEQVENDA = FILIVENDCX.NRSEQVENDA
             AND I.NRORG      = FILIVENDCX.NRORG
            JOIN PRODUTO P
              ON I.CDPRODUTO = P.CDPRODUTO
             AND I.NRORG = P.NRORG
           WHERE (((((INSTR(:CDPRODLIST, P.CDPRODUTO) > 0 ) OR (P.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF ))) OR (P.CDARVPROD LIKE :PRODNIV)) OR (:TCDPRODUTO = 'T'))
           GROUP BY I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, I.CDFILIAL, FILIVENDCX.NMFILIAL, FILIVENDCX.CDLOJA, FILIVENDCX.NMLOJA
         ) A 
     ORDER BY A.CDFILIAL, A.QTDE DESC, A.CDARVPROD
  ";
}